
local FilteredCats = {}
UnitWeapons = { aeon = {}, cybran = {}, seraphim = {}, uef = {} }

do

##########################################################################################
## --Default Function Hooks
##########################################################################################

local prevModBlueprints = ModBlueprints
function ModBlueprints(all_bps)
	prevModBlueprints(all_bps)
			
	--[[ code to insert/remove new categories into varius blueprint categories tables  --]]
    --Non-destructive add/remove for numerically-keyed blueprint tables
	
    for id, bp in all_bps.Unit do
        
		--AddCategories
        if bp.AddExtraCategories and bp.Categories then
            for cat, val in bp.AddExtraCategories do
                if val and not table.find(bp.Categories, cat) then
                    table.insert(bp.Categories, cat)
                end
            end
            bp.AddExtraCategories = nil
        end
        
		--RemoveCategories
        if bp.RemoveExtraCategories and bp.Categories then
            for cat, val in bp.RemoveExtraCategories do
                if val and table.find(bp.Categories, cat) then
                    table.removeByValue(bp.Categories, cat)
                end
            end
            bp.RemoveExtraCategories = nil
        end
        
		--AddBuildableCategory
        if bp.AddExtraBuildableCategory and bp.Economy and bp.Economy.BuildableCategory then
            for cat, val in bp.AddExtraBuildableCategory do
                if val and not table.find(bp.Economy.BuildableCategory, cat) then
                    table.insert(bp.Economy.BuildableCategory, cat)
                end
            end
            bp.AddExtraBuildableCategory = nil
        end
		
        --RemoveBuildableCategory
        if bp.RemoveExtraBuildableCategory and bp.Economy and bp.Economy.BuildableCategory then
            for cat, val in bp.RemoveExtraBuildableCategory do
                if val and table.find(bp.Economy.BuildableCategory, cat) then
                    table.removeByValue(bp.Economy.BuildableCategory, cat)
                end
            end
            bp.RemoveExtraBuildableCategory = nil
        end
						
		--Add shield Slider if we have one.
		DMS_CheckForShieldSlider(bp)
				
		--Add Command caps into a new table.. 
		DMS_SetCommandCaps(bp)
		
		--check if the unit has a shield, if so switch the energy usage.
		DMS_ShieldUnitCheck(bp)
		
		DMS_ExtractPersonalShieldMeshBlueprint(bp)
		
		DMS_CheckForMissingPrerequeEnhancements(bp)
		
		---------------------------------------------------------------------------------------------------------------

		--MISC		
		--DMS_ResetBuiltTime(bp)
		
		--i see this error alot.. stupid error.. from sorian AI not checking for a bp table in a units weapon table before looping it.
		DMS_FixSorianFireTargetLayerCapsTable(bp)
    end
	
	
    --Projectiles
    for id, bp in all_bps.Projectile do
        --AddCategories
        if bp.AddExtraCategories and bp.Categories then
            for cat, val in bp.AddExtraCategories do
                if val and not table.find(bp.Categories, cat) then
                    table.insert(bp.Categories, cat)
                end
            end
            bp.AddExtraCategories = nil
        end
        --RemoveCategories
        if bp.RemoveExtraCategories and bp.Categories then
            for cat, val in bp.RemoveExtraCategories do
                if val and table.find(bp.Categories, cat) then
                    table.removeByValue(bp.Categories, cat)
                end
            end
            bp.RemoveExtraCategories = nil
        end
    end

	---------------------------------------------------------------------------
end

##########################################################################################
## --Misc Functions
##########################################################################################

function DMS_FixSorianFireTargetLayerCapsTable(bp)
	if not bp then return end
	if bp.Weapon and table.getn(bp.Weapon) > 0 then
		for id, wepBp in bp.Weapon do
			if wepBp.FireTargetLayerCapsTable == nil and type(bp.Weapon[id]) == 'table' then 				
				bp.Weapon[id].FireTargetLayerCapsTable = {}
			end		
		end		
	end
end

function DMS_ResetBuiltTime(bp)
	if bp and bp.Economy then 
	
		bp.Economy.BuildCostEnergy = 1
        bp.Economy.BuildCostMass = 1
        bp.Economy.BuildTime = 1
	end

end

function DMS_CheckForShieldSlider(bp)
	if bp.Categories and table.find(bp.Categories, 'SHIELD') and table.find(bp.Categories, 'SHIELDRESIZER') then 
				
		if bp.Defense and bp.Defense.Shield then
			bp.Defense.Shield.ShieldRadius = bp.Defense.Shield.ShieldSize
			bp.Defense.Shield.Resizable = true
			bp.Defense.Shield.ShieldSize = 0.000000000000001
		end
						
		--Create a dummy weapon
		if not bp.Weapon then
			bp.Weapon = {}
		end
			
		local dummyweapon = {
			Label = 'ShieldRadius',
			Damage = 0,
			MaxRadius = 0.000000000000001,
			RangeCategory = 'UWRC_Indirectfire',
		}
						
		table.insert(bp.Weapon, 1, dummyweapon)
			
		--Give the unit the OVERLAYDEFENSE cat
		if not table.find(bp.Categories, 'OVERLAYDEFENSE') then
			table.insert(bp.Categories, 'OVERLAYDEFENSE')
		end
	end
end

function DMS_ShieldUnitCheck(bp)
	if not bp then return end 
	
	if bp.Defense.Shield then 
		local Shield = bp.Defense.Shield
		
		if Shield.AutomaticShield then
			--WARN('BP AutoMaticShield --> ' .. repr(bp.BlueprintId))
		end
		
		if not Shield.Hook then
			local Maintenance = bp.Economy.MaintenanceConsumptionPerSecondEnergy or 0
			local ShieldUsage = Shield.MaintenanceConsumptionPerSecondEnergy
			
			--set shield identifier.
			if not Shield.ShieldIdentifier then
				Shield.ShieldIdentifier = 'MyShield'
			end
			
			--set offsets
			if not Shield.OffsetX then
				Shield.OffsetX = 0
			end
			
			if Shield.ShieldVerticalOffset then
				if not Shield.OffsetY then
					Shield.OffsetY = Shield.ShieldVerticalOffset
				else
					Shield.ShieldVerticalOffset = Shield.OffsetY
				end
			end
			
			if not Shield.OffsetY then
				Shield.OffsetY = 0
			end
			
			if not Shield.OffsetZ then
				Shield.OffsetZ = 0
			end			
						
			--if the units maintenance is just for the shield, switch its econ to the shield econ.
			if Maintenance != 0 and not ShieldUsage or ShieldUsage == 0 then 
				bp.Economy.MaintenanceConsumptionPerSecondEnergy = 0
				Shield.MaintenanceConsumptionPerSecondEnergy = Maintenance
			end
					
			bp.Defense.Shield = Shield
		end
	end
	
	if bp.Enhancements and table.getsize(bp.Enhancements) > 0 then 
		local CheckEnh = { 'Shield', 'ShieldGeneratorField', 'PersonalShield', 'ShieldDome', 'DomeShield' }
		for id, EnhParams in bp.Enhancements do
			local Enh = EnhParams
			
			if Enh.AutomaticShield then
				--WARN('Enh AutomaticShield --> ' .. repr(bp.BlueprintId))
			end
					
			if table.find(CheckEnh, id) then
				--if the shield enh doesnt have an energy usage set it to zero.
				if not Enh.Hook then
					--set shield identifier.
					if not Enh.ShieldIdentifier then
						Enh.ShieldIdentifier = 'MyShield'
					end
					
					--set offsets
					if not Enh.OffsetX then
						Enh.OffsetX = 0
					end
					
					if Enh.ShieldVerticalOffset then
						if not Enh.OffsetY then
							Enh.OffsetY = Enh.ShieldVerticalOffset
						else
							Enh.ShieldVerticalOffset = Enh.OffsetY
						end
					end
					
					if not Enh.OffsetY then
						Enh.OffsetY = 0
					end
					
					if not Enh.OffsetZ then
						Enh.OffsetZ = 0
					end			
					
					--set the shield energy usage
					if not Enh.MaintenanceConsumptionPerSecondEnergy then 
						Enh.MaintenanceConsumptionPerSecondEnergy = 0
					end
					
					bp.Enhancements[id] = Enh
				end
			else
				--do a check for shield settings and change energy usage to zero if it doesnt have one, shield enh might be called something else.
				if EnhParams.ShieldMaxHealth and EnhParams.ShieldRechargeTime and EnhParams.ShieldRegenRate then
					if not Enh.Hook then
						--set shield identifier.
						if not Enh.ShieldIdentifier then
							Enh.ShieldIdentifier = 'MyShield'
						end
						
						--set offsets
						if not Enh.OffsetX then
							Enh.OffsetX = 0
						end
						
						if Enh.ShieldVerticalOffset then
							if not Enh.OffsetY then
								Enh.OffsetY = Enh.ShieldVerticalOffset
							else
								Enh.ShieldVerticalOffset = Enh.OffsetY
							end
						end
						
						if not Enh.OffsetY then
							Enh.OffsetY = 0
						end
						
						if not Enh.OffsetZ then
							Enh.OffsetZ = 0
						end			
						
						--set the shield energy usage
						if not Enh.MaintenanceConsumptionPerSecondEnergy then 
							Enh.MaintenanceConsumptionPerSecondEnergy = 0
						end
						
						bp.Enhancements[id] = Enh
					end
				end
			end
		end
	end
end

function DMS_SetCommandCaps(bp)
	if not bp then return end
	
	local CommandCaps = bp.General.CommandCaps
	local ToggleCaps = bp.General.ToggleCaps
	
	if CommandCaps and table.getsize(CommandCaps) > 0 then 
		if not bp.DMSBpStuff then 
			bp.DMSBpStuff = {}
		end
		
		bp.DMSBpStuff.CommandCaps = CommandCaps
	end
	
	if ToggleCaps and table.getsize(ToggleCaps) > 0 then 
		if not bp.DMSBpStuff then 
			bp.DMSBpStuff = {}
		end
		
		bp.DMSBpStuff.ToggleCaps = ToggleCaps
	end
end

function DMS_CheckForMissingPrerequeEnhancements(bp)
	if not bp then 
		return 
	elseif not bp.Enhancements then 
		return
	end
	
	local UnitEnhancements = bp.Enhancements	
	for EnhName, Params in UnitEnhancements do
		local EnhPrereq = Params.Prerequisite or nil
		
		if EnhPrereq then 
			if not UnitEnhancements[EnhPrereq] then				
				WARN('<--------------------------------------------------------->')
				WARN('--> DMS Enhancements ERROR Report <--')
				WARN('--> Enhancement Prerequisite NOT FOUND --> ' .. repr(EnhPrereq) ..' on unitid --> ' .. repr(bp.BlueprintId) .. ' on slot --> ' .. repr(Params.Slot))
				WARN('--> Removing ' .. repr(EnhName) .. ' from the units enhancements!')
				WARN('')
				
				bp.Enhancements[EnhName] = nil
			end
		end
	end
end

------------------------------------------------------------------------------------------------------------------------------------------

function DMS_ShowAllCats(bp)
	local BpCats = bp.Categories or {}
	
	for id, Cat in BpCats do
		if not table.find(FilteredCats, Cat) then 
			table.insert(FilteredCats, Cat)
		end
	end
end

function DMS_ShowAllUnitWeapons(bp, faction)
	local Weps = bp.Weapon
	local FoundWeps = {}
	
	for id, Weapon in Weps do
		local Label = Weapon.Label
		if Weapon and Label and not FoundWeps[Label] then
			if not faction then 
				LOG('FOUND FACTION Weapon --> ' .. repr(bp.General.FactionName) .. ' Label ' .. repr(Label) .. ' for unit id --> ' .. repr(bp.BlueprintId))
			elseif bp.General.FactionName and bp.General.FactionName == faction then 
				LOG('FOUND FACTION Weapon --> ' .. repr(bp.General.FactionName) .. ' Label ' .. repr(Label) .. ' for unit id --> ' .. repr(bp.BlueprintId))
			end
			FoundWeps[Label] = Weapon
		end
	end
end


##########################################################################################
## --New Mesh Effects
##########################################################################################

--do not use this on seraphim construction structure units.
function DMS_ExtractPersonalShieldMeshBlueprint(bp)
	local meshid = bp.Display.MeshBlueprint
	if not meshid then return end
	
	local meshbp = original_blueprints.Mesh[meshid]
	if not meshbp then return end
	
	local excludedCats = { 
		-- Custom unit restrictions
		'HOLOGRAM','DRONE','MINE',  
		-- Misc unit restrictions
		'POD','SATELLITE','UNTARGETABLE','PROJECTILE', 
		'OPERATION','INSIGNIFICANTUNIT', 
		'UNSELECTABLE','BENIGN',                 
	}                               
        
	-- Verify the unit type is one that we are allowed to enhance, if so retrun   
	if bp.Categories and table.getn(bp.Categories) > 0 then 
		for k, v in excludedCats do 
			if table.find(bp.Categories, v) then
				return      
			end           
		end  
	end
	
	
	local Path = Dirname(meshbp.Source)
	local bpId = string.lower(bp.BlueprintId)
	
	local faction = bp.General.FactionName or 'UEF'
	local FactionName = string.upper(string.sub(faction, 1, 1)) .. string.lower(string.sub(faction, 2,-1))
	local LookupName, SecondaryName, ShaderName, SecondaryName, Scrolling, LODCutoff
	
	if FactionName == 'Aeon' then 
		LookupName = '/effects/entities/PhaseShield/PhaseShieldLookup.dds'
        ShaderName = 'PhaseShield'
        LODCutoff = 195
	elseif FactionName == 'Cybran' then 
		LookupName = '/effects/entities/PhaseShield/PhaseShieldLookup.dds'
        ShaderName = 'PhaseShield'
        LODCutoff = 195
	elseif FactionName == 'Seraphim' then
		LookupName = '/textures/environment/Falloff_seraphim_lookup.dds'
		SecondaryName = '/effects/entities/PhaseShield/PhaseShieldLookup.dds'
		LODCutoff = 195
        Scrolling = true
        ShaderName = 'SeraphimPersonalShield'
	   	   
	elseif FactionName == 'Uef' then 
		LookupName = '/effects/entities/PhaseShield/PhaseShieldLookup.dds'
        ShaderName = 'PhaseShield'
        LODCutoff = 195
	else
		LookupName = '/effects/entities/PhaseShield/PhaseShieldLookup.dds'
        ShaderName = 'PhaseShield'
        LODCutoff = 195
	end
							
	local personalshieldmeshbp = table.deepcopy(meshbp)
	personalshieldmeshbp.Merge = true
	if personalshieldmeshbp.LODs then
		for i,lod in personalshieldmeshbp.LODs do

		--	lod.MeshName = Path..'/'..bpId..'_lod0.scm'
		--	lod.AlbedoName = Path..'/'..bpId..'_albedo.dds'
		--	lod.NormalsName = Path..'/'..bpId..'_normals.dds'
		--	lod.SpecularName = Path..'/'..bpId..'_specteam.dds'
			
			lod.LookupName = LookupName
			lod.SecondaryName = SecondaryName
			lod.ShaderName = ShaderName
			lod.LODCutoff = LODCutoff
			lod.Scrolling = Scrolling
		end
	end
	
	personalshieldmeshbp.BlueprintId = meshid .. '_personalshield'
	bp.Display.PersonalShieldMeshBlueprint = personalshieldmeshbp.BlueprintId
	MeshBlueprint(personalshieldmeshbp)
end



end -- end of none destructive hook
